@@ -206,9 +206,20 @@ class DisplayEmbryo():
 
         self.selected_adata = adata[mask].copy()
         points.shown = mask
+        self.mask_filter = mask
         points.refresh()
         
     def tissue_tab(self):
+        viewer = self.viewer
+        def celltype_coloring():
+            points = viewer.layers.selection.active
+            adata = self.embryo.adata
+            tissue_types = adata.obs[self.embryo.tissue_name].astype(str).values
+            self.viewer.dims.ndisplay = 3
+            position = adata.obsm[self.embryo.coordinate_3d_key]
+            points.features = adata.obs
+            points.data = position
+            points.face_color = [self.tissue_color_map[t] for t in tissue_types]
         def select_tissue():
             self.tissue_select = widgets.Select(
                 choices=self.embryo.all_tissues,
@@ -219,13 +230,18 @@ class DisplayEmbryo():
                 call_button="Select Tissue",
                 layout="vertical",
             )
+            run_celltype_coloring = widgets.FunctionGui(
+                celltype_coloring,
+                call_button="Color by Tissue",
+                layout="vertical",
+            )
             run_show_flatten = widgets.FunctionGui(
                 self.show_flatten,
                 call_button="Show in Flatten",
                 layout="vertical",
             )
             select_tissue_container = widgets.Container(
-                widgets=[self.tissue_select, run_tissue_filter, run_show_flatten],
+                widgets=[self.tissue_select, run_tissue_filter,run_celltype_coloring, run_show_flatten],
                 layout="vertical",
             )
             return select_tissue_container
@@ -237,7 +253,7 @@ class DisplayEmbryo():
         return tissue_tab
     
     def show_flatten(self):
-        adata = self.selected_adata
+        adata = self.embryo.adata
         points = self.viewer.layers.selection.active
         if points is None or points.as_layer_data_tuple()[-1] != "points":
             error_points_selection(show=True)
@@ -251,7 +267,7 @@ class DisplayEmbryo():
             x_flatten = adata.obs['y_flatten'].values
             y_flatten = adata.obs['x_flatten'].values
             xy_flatten = np.column_stack((x_flatten, y_flatten))
-            
+        shown_ori = points.shown
         colors = points.face_color
         self.viewer.add_points(
             xy_flatten,
@@ -259,6 +275,7 @@ class DisplayEmbryo():
             face_color=colors,
             features=adata.obs,
             name='flatten',
+            shown=shown_ori,
         )
 
     def slice_tab(self):
@@ -601,9 +618,11 @@ class DisplayEmbryo():
             gene_exp_norm = (gene_exp - min_exp) / (max_exp - min_exp)
             gene_exp_norm = np.clip(gene_exp_norm, 0.0, 1.0)
             mask = (gene_exp_norm >= min_v) & (gene_exp_norm <= max_v)
-            points.shown = mask
+            shown_ori = self.mask_filter
+            mask_final = np.logical_and(shown_ori, mask)
+            points.shown = mask_final
             points.refresh()
-            self.threshold_output.value = f"Showing {mask.sum()} cells out of {len(mask)}"
+            self.threshold_output.value = f"Showing {mask_final.sum()} cells out of {len(mask_final)}"
 
         def show_gene():
             """
@@ -776,17 +795,16 @@ class DisplayEmbryo():
     
     def two_genes_tab(self):
         viewer = self.viewer
-
+        adata = self.embryo.adata
         def show_two_genes():
             """
             Colour cells according to their gene expression
             """
-            adata = self.selected_adata
             points = viewer.layers.selection.active
             gene_1 = self.gene_1.value
             gene_2 = self.gene_2.value
-            gene_exp_1 = safe_toarray(self.selected_adata[:, gene_1].X)[:,0]
-            gene_exp_2 = safe_toarray(self.selected_adata[:, gene_2].X)[:,0]
+            gene_exp_1 = safe_toarray(adata[:, gene_1].X)[:,0]
+            gene_exp_2 = safe_toarray(adata[:, gene_2].X)[:,0]
             min_exp_1, max_exp_1 = gene_exp_1.min(), gene_exp_1.max()
             min_exp_2, max_exp_2 = gene_exp_2.min(), gene_exp_2.max()
             with np.errstate(divide='ignore', invalid='ignore'):
@@ -806,7 +824,6 @@ class DisplayEmbryo():
             points.refresh()
 
         def threshold_2gene():
-            adata = self.selected_adata
             points = viewer.layers.selection.active
             if points is None or points.as_layer_data_tuple()[-1] != "points":
                 error_points_selection(show=True)
@@ -828,8 +845,10 @@ class DisplayEmbryo():
                 max_v_1, min_v_1 = min_v_1, max_v_1
             mask = (gene_exp_norm_1 >= min_v_1) & (gene_exp_norm_1 <= max_v_1) & \
                    (gene_exp_norm_2 >= min_v_2) & (gene_exp_norm_2 <= max_v_2)
-            points.shown = mask
-            self.threshold_output.value = f"Showing {mask.sum()} cells out of {len(mask)}"
+            shown_ori = self.mask_filter
+            mask_final = np.logical_and(shown_ori, mask)
+            points.shown = mask_final
+            self.threshold_output_2g.value = f"Showing {mask_final.sum()} cells out of {len(mask_final)}"
             points.refresh()
 
         def container():
@@ -966,8 +985,10 @@ class DisplayEmbryo():
             mask = (gene_exp_norm_1 >= min_v_1) & (gene_exp_norm_1 <= max_v_1) & \
                    (gene_exp_norm_2 >= min_v_2) & (gene_exp_norm_2 <= max_v_2) & \
                    (gene_exp_norm_3 >= min_v_3) & (gene_exp_norm_3 <= max_v_3)
-            points.shown = mask
-            self.threshold_output_3g.value = f"Showing {mask.sum()} cells out of {len(mask)}"
+            shown_ori = self.mask_filter
+            mask_final = np.logical_and(shown_ori, mask)
+            points.shown = mask_final
+            self.threshold_output_3g.value = f"Showing {mask_final.sum()} cells out of {len(mask_final)}"
             points.refresh()
 
         def container():
@@ -1115,8 +1136,8 @@ class DisplayEmbryo():
         def compute_moran():
             adata = self.selected_adata
             geneslist1, num1 = sc.pp.filter_genes(self.selected_adata, min_counts=self.min_counts.value,inplace=False)
-            geneslist2, num2 = sc.pp.filter_genes(self.selected_adata, min_cells=self.min_cells.value, inplace=False)
-            geneslist3, num3 = sc.pp.filter_genes(self.selected_adata, max_cells=self.max_cells.value, inplace=False)
+            geneslist2, num2 = sc.pp.filter_genes(self.selected_adata, min_cells=self.min_cells.value*self.embryo.adata.obs.shape[0], inplace=False)
+            geneslist3, num3 = sc.pp.filter_genes(self.selected_adata, max_cells=self.max_cells.value*self.embryo.adata.obs.shape[0], inplace=False)
             geneslist = [a and b and c for a, b, c in zip(geneslist1, geneslist2, geneslist3)]
             adata = self.selected_adata[:, geneslist]
             self.threshold_output_moran.value = f"computer Moran's I for {len(adata)} cells"
@@ -1131,33 +1152,36 @@ class DisplayEmbryo():
             self.moran_gene.choices = choice
         
         def plot_moran():
-            points = viewer.layers.selection.active
-            if isinstance (self.moran_gene.value, str):
-                gene = self.moran_gene.value.split(' (')[0]
+            if self.moran_gene.value == 'compute Moran\'s I first' or len(self.moran_gene.value) == 0:
+                return
             else:
-                gene = self.moran_gene.value[0].split(' (')[0]
-            gene_exp = safe_toarray(self.embryo.adata[:, gene].X)[:,0]
-            min_exp, max_exp = gene_exp.min(), gene_exp.max()
-            gene_exp_norm = (gene_exp - min_exp) / (max_exp - min_exp)
-            features = {}
-            features[f'gene_exp_{gene}'] = gene_exp
-            colors = ALL_COLORMAPS[self.cmap.value]
-            points.face_color = colors.map(gene_exp_norm)
-            points.features = features
-            points.refresh()
+                points = viewer.layers.selection.active
+                if isinstance (self.moran_gene.value, str):
+                    gene = self.moran_gene.value.split(' (')[0]
+                else:
+                    gene = self.moran_gene.value[0].split(' (')[0]
+                gene_exp = safe_toarray(self.embryo.adata[:, gene].X)[:,0]
+                min_exp, max_exp = gene_exp.min(), gene_exp.max()
+                gene_exp_norm = (gene_exp - min_exp) / (max_exp - min_exp)
+                features = {}
+                features[f'gene_exp_{gene}'] = gene_exp
+                colors = ALL_COLORMAPS[self.cmap.value]
+                points.face_color = colors.map(gene_exp_norm)
+                points.features = features
+                points.refresh()
 
         def show_moran():
             self.min_counts = widgets.FloatSpinBox(
                 value=0.0, min=0.0, max=100.0, step=0.01,
                 label="Minimum count for per gene"
             )
-            self.min_cells = widgets.SpinBox(
-                value=0, min=0, max=self.embryo.adata.obs.shape[0], step=1,
-                label="Minimum n_cells for per gene"
+            self.min_cells = widgets.FloatSpinBox(
+                value=0, min=0, max=1, step=0.01,
+                label="Floor precentage of n_cells"
             )
-            self.max_cells = widgets.SpinBox(
-                value=self.embryo.adata.obs.shape[0], min=0, max=self.embryo.adata.obs.shape[0], step=1,
-                label="Maximum n_cells for per gene"
+            self.max_cells = widgets.FloatSpinBox(
+                value=1, min=0, max=1, step=0.01,
+                label="Ceiling precentage of n_cells"
             )
             self.threshold_output_moran = widgets.Label(value="")
             run_moran = widgets.FunctionGui(
@@ -1171,11 +1195,7 @@ class DisplayEmbryo():
                 label="Select a gene\nto see exp",
                 allow_multiple=False,
             )
-            run_plot_moran = widgets.FunctionGui(
-                plot_moran,
-                call_button="Show chosed gene exp",
-                layout="vertical",
-            )
+            self.moran_gene.changed.connect(plot_moran)
             moran_container = widgets.Container(
                 widgets=[
                     self.min_counts,
@@ -1184,7 +1204,6 @@ class DisplayEmbryo():
                     self.threshold_output_moran,
                     run_moran,
                     self.moran_gene,
-                    run_plot_moran,
                 ],
                 layout="vertical",
             )
